<?php

namespace Snaju\Inception;

use Dotenv\Dotenv;
use Snaju\Inception\Event\Types\OnRouteStart;
use Snaju\Inception\Exception\AppExitException;
use Snaju\Inception\Exception\Exception;
use Snaju\Inception\Exception\ForbiddenException;
use Snaju\Inception\Exception\InvalidKeyException;
use Snaju\Inception\ORM\ORM;
use Snaju\Inception\Route\Loaders\Func\FuncRouteLoader;
use Snaju\Inception\Route\Loaders\Twig\TwigRouteLoader;
use Snaju\Inception\Http\Middleware\MiddlewareRegistry;
use Snaju\Inception\Http\Request\WebRequest;
use Snaju\Inception\Route\RouteManager;
use Snaju\Inception\Util\ClassAsyncLoader;
use Snaju\Inception\Util\Loader;
use Snaju\Inception\Exception\Handlers\ExceptionHandler;


class InceptionCore
{

    private static string $base = __DIR__;

    private static string $appRoot;

    public static MiddlewareRegistry $middleware;

    private static WebRequest $request;

    private static $ENV;

    private static ExceptionHandler $exceptionHandler;

    public static function getBase(): string
    {
        return self::$base;
    }

    public static function initBase($base = __DIR__)
    {
        self::$base = $base;
    }

    /**
     * @return WebRequest
     */
    public static function &getRequest(): WebRequest
    {
        return self::$request;
    }

    /**
     * @return mixed
     */
    public static function &getENV()
    {
        return self::$ENV;
    }

    public static function init($base): void
    {
        self::$ENV = Dotenv::createImmutable($base);
        self::$ENV->safeLoad();
        self::initBase($base);
        self::$request = new WebRequest();
        self::$middleware = new MiddlewareRegistry();
        self::$exceptionHandler = new ExceptionHandler();

        // Include and declare global helper functions
        include_once 'Util/GlobalHelpers.php';

        // Initialize the route tree, add default loaders and init route manager
        RouteManager::initTree();

        // Add default loaders: (Twig templates, Functional routes, & Plugins)
        if ((bool)config('app.useTwigRendering')) {
            RouteManager::addLoader(new TwigRouteLoader());
        }

        RouteManager::addLoader(new FuncRouteLoader());
        RouteManager::init();


        // Set the global exception handler
        set_exception_handler(self::$exceptionHandler);


        if (config('app.rootDirectory') != null) {
            self::$appRoot = $base . DIRECTORY_SEPARATOR . config('app.rootDirectory');
        } else {
            // Not set check if /app exist
            if (file_exists($base . DIRECTORY_SEPARATOR . "app")) {
                // It does exist, this will be the root
                self::$appRoot = $base . DIRECTORY_SEPARATOR . "app";
            } else {
                throw new \Exception("Failed to locate app root in (" . $base . DIRECTORY_SEPARATOR . "app)");
            }
        }


//        RouteManager::initTree();

//        InjectionManager::init();
//
        $loader = new Loader(self::$appRoot, '/.*\.php/mi', false);
        foreach ($loader->getMatches() as $m) {
            require_once $m;
            ClassAsyncLoader::fromFile($m);
        }

//        InjectionManager::init();




//        echo "should use twig rendering::: " . (bool)Config::get('app.useTwigRendering');


//        // Add default loaders: (Twig templates, Functional routes, & Plugins)
//        if ((bool)config('app.useTwigRendering')) {
//            RouteManager::addLoader(new TwigRouteLoader());
//        }
//
//        RouteManager::addLoader(new FuncRouteLoader());
        // TODO: add plugin loader to load and register routes for plugins @ "/plugins"


//        try {
        ob_start();
        session_start();


//            self::$request = new WebRequest();
//            self::$middleware = new MiddlewareRegistry();

//            RouteManager::init();


//            echo '<pre>';
//            print_r(self::$request);
//            echo '</pre>';
//
//            exit;


        // Add the location for the ORM to load models
        if (file_exists(self::$appRoot . DIRECTORY_SEPARATOR . config('database.modelsDirectory'))) {
            ORM::addModelLocation(self::$appRoot . DIRECTORY_SEPARATOR . config('database.modelsDirectory'));
        }

        // Setup and initialize the ORM
        if (config('database.enableORM')) {
            // Setup the Database
            $dbConfig = ORM::init();
            $dbConfig->setAutoGenerateProxyClasses(true);
            $dbConfig->setProxyDir(self::$base . "/tmp");

            ORM::connect([
                "dbname" => env('DB_NAME'),
                "user" => env('DB_USER'),
                "password" => env('DB_PASS'),
                "host" => env('DB_HOST'),
                "port" => env("DB_PORT"),
                "driver" => "pdo_mysql",
            ], $dbConfig);

            if (config('database.autoSyncing')) {
                ORM::updateSchema();
            }
//                ORM::setupCLI();
        }

        // Last part, execute the call to the route
        // This will find and match to a registered route, process the request through
        // the request handler and generate an appropriate response to the request
        (new OnRouteStart(self::$request->getUri()->getPath()))->call();
//        }
//            catch(\Exception $exception) {
//                echo "exception thrown from InceptionCore.php";
//                echo "<pre>";
//                print_r($exception);
//                echo "</pre>";
//            }

    }

}