<?php

namespace Snaju\Inception\ORM;

use Doctrine\ORM\Configuration;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\ClassMetadata;
use Doctrine\ORM\Tools\SchemaTool;
use Doctrine\ORM\Tools\Setup;

/**
 * Class ORM
 * @package inception\orm
 */
class ORM
{
    /**
     * @var EntityManager
     */
    public static $EntityManager;

    /**
     * @var array
     */
    private static $modelLocations = [];
    /**
     * @var bool
     */
    private static $devMode = false;
    /**
     * @var bool
     */
    private static $generateSchema = false;

    /**
     * @return \Doctrine\ORM\Configuration
     */
    public static function init(): Configuration
    {
        return Setup::createAnnotationMetadataConfiguration(self::$modelLocations, self::$devMode, null, null, false);
    }

    public static function getClasses()
    {
        $models = [];
        $classes = self::$EntityManager->getMetadataFactory()->getAllMetadata();
        foreach ($classes as $class) {
            if ($class instanceof ClassMetadata) {
                $models[] = new $class->name();
            }
        }

        return $models;
    }

    public static function flush($objs = [])
    {
        if (count($objs) == 0) {
            self::$EntityManager->flush();

        } else {
            self::$EntityManager->flush($objs);
        }
    }

    public static function connect(array $connection, Configuration $configuration): EntityManager
    {
        self::$EntityManager = EntityManager::create($connection, $configuration);
        return self::$EntityManager;
    }

    public static function createSchema()
    {
        if (self::isAlive()) {
            $tool = new SchemaTool(self::$EntityManager);
            $classes = self::$EntityManager->getMetadataFactory()->getAllMetadata();
            $tool->createSchema($classes);
        } else {
            throw new \Exception("EntityManager not set, cannot make schema");
        }
    }

    public static function updateSchema()
    {
        if (self::isAlive()) {
            $tool = new SchemaTool(self::$EntityManager);
            $classes = self::$EntityManager->getMetadataFactory()->getAllMetadata();
            $tool->updateSchema($classes);
        } else {
            throw new \Exception("EntityManager not set, cannot update schema");
        }
    }

    public static function isAlive()
    {
        return (self::$EntityManager != null && self::$EntityManager instanceof EntityManager);
    }

    /**
     * @param $dir
     */
    public static function addModelLocation($dir)
    {
        self::$modelLocations[] = $dir;
    }

    /**
     * @return mixed
     */
    public static function getEntityManager()
    {
        return self::$EntityManager;
    }

    /**
     * @param mixed $EntityManager
     */
    public static function setEntityManager($EntityManager)
    {
        self::$EntityManager = $EntityManager;
    }

    /**
     * @return array
     */
    public static function getModelLocations(): array
    {
        return self::$modelLocations;
    }

    /**
     * @param array $modelLocations
     */
    public static function setModelLocations(array $modelLocations)
    {
        self::$modelLocations = $modelLocations;
    }

    /**
     * @return bool
     */
    public static function isDevMode(): bool
    {
        return self::$devMode;
    }

    /**
     * @param bool $devMode
     */
    public static function setDevMode(bool $devMode)
    {
        self::$devMode = $devMode;
    }

    /**
     * @return bool
     */
    public static function isGenerateSchema(): bool
    {
        return self::$generateSchema;
    }

    /**
     * @param bool $generateSchema
     */
    public static function setGenerateSchema(bool $generateSchema)
    {
        self::$generateSchema = $generateSchema;
    }

}

?>