<?php

namespace Snaju\Inception\Route;

use Composer\Autoload\ClassLoader;
use Doctrine\Common\Annotations\AnnotationReader;
use Snaju\Inception\Annotation\InceptionRoute;
use Snaju\Inception\Exception\RouteNotFoundException;
use Snaju\Inception\Util\AnnotationHelper;

class RouteManager
{

    /**
     * @var RouteTree
     */
    private static RouteTree $tree;

    /**
     * @var array
     */
    private static array $loaders = [];

    public static ?string $lastMatchedRoutePart;

    public static function register(Route $route)
    {
        self::$tree->pushPath($route->getFullPath(), $route);
    }

    public static function &findRoute($path)
    {
        $node = self::$tree->searchPath($path);
        if ($node instanceof RouteNode) {
            if ($node->getRouteObject() != null) {
                self::$lastMatchedRoutePart = $node->getRoutePart();
                return $node->getRouteObject();
            }
            throw new RouteNotFoundException("No Route Found with path {$path}", 404);
        }

        throw new RouteNotFoundException("Route Not Found with path {$path}. Invalid RouteNode received.", 404);
    }

    public static function attachRouteFile(string $path, callable $func)
    {
        $route = self::findRoute($path);
        if ($route != null) {
            $route->attach(new RouteFile($func));
        }
    }

    public static function initTree()
    {
        self::$tree = new RouteTree(new RouteNode(null, null, null));
    }

    public static function init()
    {
        $classes = AnnotationHelper::findClassesWithAnnotation(InceptionRoute::class);
        foreach ($classes as $class) {
            if ($class instanceof \ReflectionClass) {
                $annon = AnnotationHelper::getClassAnnotation($class, InceptionRoute::class);
                if ($annon instanceof InceptionRoute) {
                    $obj = $class->newInstance($annon->attachToPath);
                    self::$tree->pushPath($annon->attachToPath, $obj);
                }
            }
        }

        // Call the loaders
        foreach (self::$loaders as $loader) {
            if ($loader instanceof RouteLoader) {
                $loader->loadRoutes();
            }
        }
    }

    public static function addLoader(RouteLoader $loader)
    {
        self::$loaders[] = $loader;
    }

    /**
     * @return RouteTree
     */
    public static function getTree(): RouteTree
    {
        return self::$tree;
    }

}