<?php

namespace Snaju\Inception\Route;

class RouteNode
{

    /**
     * @var array
     */
    private array $children = [];

    /**
     * @var RouteNode
     */
    private $parent;

    /**
     * @var string
     */
    private $routePart;

    /**
     * @var Route
     */
    private $routeObject;

    /**
     * @var array
     */
    private array $data = [];

    /**
     * RouteNode constructor.
     * @param $parent
     * @param $routePart
     * @param null $routeObject
     */
    public function __construct($parent, $routePart, $routeObject = null)
    {
        $this->parent = $parent;
        $this->routePart = $routePart;
        $this->routeObject = $routeObject;
    }

    /**
     * @return array
     */
    public function getChildren(): array
    {
        return $this->children;
    }

    /**
     * @param array $children
     */
    public function setChildren(array $children): void
    {
        $this->children = $children;
    }

    /**
     * @return mixed
     */
    public function getParent()
    {
        return $this->parent;
    }

    /**
     * @param mixed $parent
     */
    public function setParent($parent): void
    {
        $this->parent = $parent;
    }

    /**
     * @return mixed
     */
    public function getRoutePart()
    {
        return $this->routePart;
    }

    /**
     * @param mixed $routePart
     */
    public function setRoutePart($routePart): void
    {
        $this->routePart = $routePart;
    }

    public function addChild(RouteNode $node)
    {
        if (!$this->hasChild($node->getRoutePart())) {
            $node->setParent($this);
            $this->children[] = $node;
        }

        return $this;
    }

    public function removeChild(string $stringPart)
    {
        foreach ($this->children as $i => $child) {
            if ($child instanceof RouteNode) {
                if (strtolower($child->routePart) == strtolower($stringPart)) {
                    $this->children[$i] = null;
                    unset($this->children[$i]);
                }
            }
        }
    }

    public function getChild(string $part)
    {
        foreach ($this->children as $child) {
            if ($child instanceof RouteNode) {
                if (strtolower($child->getRoutePart()) == strtolower($part)) {
                    return $child;
                } else if (preg_match('/\{(.*?)\}/mi', $child->getRoutePart(), $m)) {
                    $this->routeObject->data[$m[1]] = $part;
                    return $child;
                }
            }
        }

        return null;
    }

    public function printOut()
    {
        echo "(Node: " . $this->routePart . " (<br/>";
        foreach ($this->children as $child) {
            if ($child instanceof RouteNode) {
                $child->printOut();
            }
        }
        echo ")<br/>";
    }

    public function getArray()
    {
        $ch = [];
        foreach ($this->children as $c) {
            if ($c instanceof RouteNode) {
                $ch[] = $c->getArray();
            }
        }

        return [
            "part" => $this->getRoutePart(),
            "isRoute" => ($this->getRouteObject() != null),
            "children" => $ch
        ];
    }

    public function hasChildren()
    {
        return (count($this->children) > 0);
    }

    public function hasChild(string $part)
    {
        $n = $this->getChild($part);
        return ($n instanceof RouteNode);
    }

    public function getAllParents($parts = [])
    {
        $parts[] = $this->getRoutePart();
        $parent = $this->getParent();
        if ($parent instanceof RouteNode) {
            $parts = $parent->getAllParents($parts);
        }

        return $parts;
    }

    public function getAllParentsObjects($parents = [])
    {
        $parents[] = $this;
        $parent = $this->getParent();
        if ($parent instanceof RouteNode) {
            $parents = $parent->getAllParentsObjects($parents);
        }

        return $parents;
    }

    public function getFullPath()
    {
        $parents = $this->getAllParents();
        $parents = array_reverse($parents);

        $s = "";
        foreach ($parents as $parent) {
            $parent = str_replace("/", "", $parent);
            if (!empty($parent)) {
                $s .= "/$parent";
            }
        }

        if (empty($s)) {
//            $s = "/";
        }

        return $s;
    }

    /**
     * @return Route|null
     */
    public function &getRouteObject(): ?Route
    {
        return $this->routeObject;
    }

    /**
     * @param Route $routeObject
     */
    public function setRouteObject(Route $routeObject): void
    {
        $this->routeObject = $routeObject;
    }

    /**
     * @return array
     */
    public function getData(): array
    {
        return $this->data;
    }

}

?>